import ij.*;
import ij.io.*;
import ij.gui.*;
import ij.process.*;
import ij.plugin.frame.*;
import ij.plugin.filter.PlugInFilter;
import java.awt.*;
import java.io.*;
import java.util.*;
import java.awt.List;
import java.awt.event.*;
import javax.swing.*;
import UroIrmPackage.*;

/**
This program helps to manage Rois of the stack for the test Uro-Irm.	
*/
public class UroIrmARoiManager_ extends PlugInFrame implements ActionListener, ItemListener,UroIrmMessage {

	Panel panel;
	static Frame instance;
	java.awt.List list;
	Hashtable rois = new Hashtable();
	final String menu_separator = "------------";
	ImagePlus imp;

	public UroIrmARoiManager_() {
		super("UroIrm ROI Manager_");
		if (instance!=null) {
			instance.toFront();
			return;
		}
		instance = this;
		imp = getImage();
		if ( imp == null){
			instance = null; 
			return; 
		}
		setLayout(new FlowLayout(FlowLayout.CENTER,5,5));
		int rows = 18;
		list = new List(rows, false);
		list.add("012345678901234567");
		list.addItemListener(this);
		add(list);
		panel = new Panel();
		panel.setLayout(new GridLayout(13, 1, 5, 5));
		addButton(ADD);
		addButton(DELETE);
		panel.add(new Label(menu_separator,Label.CENTER));
		addButton(OPEN);
		addButton(OPENALL);
		addButton(SAVE_ROI);
		addButton(SAVE_IMAGE_ROI);
		addButton(SAVE_EXAMEN);
		panel.add(new Label(menu_separator,Label.CENTER));
		addButton(MEASURE);
		addButton(RENAME);
		addButton(UROIRM);
		add(panel);
		
		pack();
		list.delItem(0);
		show();
	}
	
	void addButton(String label) {
		Button b = new Button(label);
		b.addActionListener(this);
		panel.add(b);
	}

	public void actionPerformed(ActionEvent e) {
		String label = e.getActionCommand();
		if (label==null)
			return;
		String command = label;
		if (command.equals(ADD))
			add();
		else if (command.equals(DELETE))
			delete();
		else if (command.equals(OPEN))
			open();
		else if (command.equals(OPENALL))
			openAll();
		else if (command.equals(SAVE_ROI))
			saveRoi();
		else if (command.equals(SAVE_IMAGE_ROI))
			saveImageRoi();
		else if (command.equals(SAVE_EXAMEN))
			saveExamen();
		else if (command.equals(MEASURE))
			measure();
		else if ( command.equals(UROIRM))
			runUroIrm();
		else if ( command.equals(RENAME))
			reName();
	}
	
	public void itemStateChanged(ItemEvent e) {
		int index = 0;
        try {index = Integer.parseInt(e.getItem().toString());}
        catch (NumberFormatException ex) {}
		restore(index);
		
	}

	void add() {
		if (imp==null)
			return;
		Roi roi = imp.getRoi();
		if (roi==null) {
			error(msg_err_no_roi);
			return;
		}
		String type = null;
		switch (roi.getType()) {
			case Roi.RECTANGLE: type ="Rectangle"; break;
			case Roi.OVAL: type = "Oval"; break;
			case Roi.POLYGON: type = "Polygon"; break;
			case Roi.FREEROI: type = "Freehand"; break;
			case Roi.TRACED_ROI: type = "Traced"; break;
			case Roi.LINE: type = "Line"; break;
			case Roi.POLYLINE: type = "Polyline"; break;
			case Roi.FREELINE: type = "Freeline"; break;
		}
		if (type==null)
			return;

		//String label = type+" ("+(r.x+r.width/2)+","+(r.y+r.height/2)+")";
		Rectangle r = roi.getBoundingRect();
		String label = type+(r.x+r.width/2)+"-"+(r.y+r.height/2);
		label = newNameDialog(label);
		if (label == null)
			return;
		list.add(label);
		rois.put(label, roi.clone());
	}
	
	boolean delete() {
		if (list.getItemCount()==0)
			return error(msg_err_roi_list_empty);
		int index[] = list.getSelectedIndexes();
		if (index.length==0)
			return error(msg_err_no_roi_selected);
		for (int i=index.length-1; i>=0; i--) {
			rois.remove(list.getItem(index[i]));
			list.delItem(index[i]);
		}
		return true;
	}
	
	boolean restore(int index) {
		String label = list.getItem(index);
		Roi roi = (Roi)rois.get(label);
		ImagePlus imp = getImage();
		if (imp==null)
			return false;
		Rectangle r = roi.getBoundingRect();
		if (r.x+r.width>imp.getWidth() || r.y+r.height>imp.getHeight())
			return error("This ROI does not fit the current image.");
		imp.setRoi(roi);
		return true;
	}
	
	void open() {
		Macro.setOptions(null);
		OpenDialog od = new OpenDialog("Open ROI...", "");
		String directory = od.getDirectory();
		String name = od.getFileName();
		if (name==null)
			return;
		String path = directory + name;
		Opener o = new Opener();
		Roi roi = o.openRoi(path);
		if (roi==null) 
			return ;
		name = name.trim().substring(0,name.trim().length()-4);
		if ( rois.containsKey(name) ) {
			IJ.error(msg_err_name_exists + "\n" + name);
			name = newNameDialog(name);
			if (name == null)
				return;
		}
		list.add(name);
		rois.put(name, roi);
	}	

	void openAll() {
		Macro.setOptions(null);
		Macro.setOptions(null);
		OpenDialog od = new OpenDialog(msg_select_file, "");
		if (od.getFileName()==null) return;
		String dir  = od.getDirectory();
		String[] files = new File(dir).list();
		if (files==null) return;
		for (int i=0; i<files.length; i++) {
			File f = new File(dir+files[i]);
			if (!f.isDirectory() && files[i].endsWith(".roi")) {
                			Roi roi = new Opener().openRoi(dir+files[i]);
  				if (roi!=null) {
					String name = files[i].trim();
					name = name.substring(0,name.length()-4);
					if ( rois.containsKey(name) ) {
						IJ.error(msg_err_name_exists +"\n" + name);
						name = newNameDialog(name);
						if (name == null)
							return;
					}
					list.add(name);
					rois.put(name, roi);
				}
			}
		}
	}
	
	boolean saveRoi() {
		if (list.getItemCount()==0)
			return error(msg_err_roi_list_empty);
		int index = list.getSelectedIndex();
		if (index < 0)
			return error(msg_err_no_roi_selected);
		String name = list.getItem(index);
		Macro.setOptions(null);
		SaveDialog sd = new SaveDialog("Save ROI...", name, ".roi");
		name = sd.getFileName();
		if (name == null)
			return false;
		String dir = sd.getDirectory();
		IJ.run("ROI...", "path='"+dir+name+"'");
		return true;
	}

	Frame getNewFrame(){
		Frame frame = new Frame();
		ImageJ ij = IJ.getInstance();
		if (ij!=null) {
			Image img = ij.getIconImage();
			if (img!=null) frame.setIconImage(img);
		}
		return frame;
	}

	boolean saveImageRoi() {
		if (list.getItemCount()==0)
			return error(msg_err_roi_list_empty);
		int index = list.getSelectedIndex();
		if (index < 0)
			return error(msg_err_no_roi_selected);
		String label = list.getItem(index);
		Roi roi = (Roi)rois.get(label);
		SaveDialog sd = new SaveDialog("Save Examen...", label,".jpg" );
		if ( sd.getFileName() == null)
			return false;
		String path = sd.getDirectory()+ sd.getFileName();
		saveImageRoiJPEG(roi,label,path);
		return true;
	}

	boolean saveImageRoiJPEG(Roi roi,String roiName, String path ){
		Color colorRoi = imp.getRoi().getColor();
		ImagePlus imp2 = new ImagePlus("ImageOf"+roiName ,imp.getProcessor().duplicate().convertToRGB());
		imp2.setRoi(roi);
		ImageProcessor ip2 = imp2.getProcessor();
		ip2.setColor(colorRoi);
		roi.drawPixels();
		try {
			path = path.trim();
			if ( !path.endsWith(".jpg"))
				path += ".jpg";
			OutputStream output = new BufferedOutputStream(new FileOutputStream(path));
			JpegEncoder encoder = new JpegEncoder(imp2.getImage(), JpegEncoder.getQuality(), output);
			encoder.Compress();
			output.close();
		}
		catch (IOException e) {
			IJ.write(msg_err_compression_jpeg);
			return false;
		}
		imp2.flush();
		imp2=null;
		return true;
	}
	
	boolean saveExamen() {
		if (list.getItemCount()==0)
			return error(msg_err_roi_list_empty);
		SaveDialog sd; 
		do {
			IJ.showMessage(msg_question_save_rep_roi);
			sd = new SaveDialog("Save Examen...", "", "");
			if (sd.getFileName() == null )
				return false;
		}while ( (new File(sd.getFileName())).exists() );
	
		String dir = sd.getDirectory()+  sd.getFileName()+ Prefs.separator ;
		if ( !(new File(dir)).mkdir()){
			IJ.error("le rpertoire n'a pas pu tre cr");
			return false;
		}
		Roi roiCurrent = imp.getRoi();
		IJ.showProgress(0);
		int count = 0 , total = rois.size();
		for ( Enumeration iter = rois.keys();iter.hasMoreElements();){
			String label  = (String)iter.nextElement();
			Roi roi = (Roi)rois.get(label);
			saveImageRoiJPEG(roi,label,dir+"ImageOf"+label);
			imp.killRoi();
			imp.setRoi(roi);
			IJ.run("ROI...", "path='"+dir+label+".roi"+"'");
			IJ.showStatus(SAVE + label );
			IJ.showProgress((double)count/total);
		}
		imp.killRoi();
		imp.setRoi(roiCurrent);
		IJ.showProgress(1);
		return true;
	}
	
	boolean measure() {
		ImagePlus imp = getImage();
		if (imp==null)
			return false;
		int[] index = list.getSelectedIndexes();
		if (index.length==0)
			return error(msg_err_no_roi_selected);
		
		int setup = IJ.setupDialog(imp, 0);
		if (setup==PlugInFilter.DONE)
			return false;
		int nSlices = setup==PlugInFilter.DOES_STACKS?imp.getStackSize():1;
		int currentSlice = imp.getCurrentSlice();
		for (int slice=1; slice<=nSlices; slice++) {
			imp.setSlice(slice);
			for (int i=0; i<index.length; i++) {
				if (restore(index[i]))
					IJ.run("Measure");
				else
					break;
			}
		}
		imp.setSlice(currentSlice);
		if (index.length>1)
			IJ.run("Select None");
		return true;
	}	

	ImagePlus getImage() {
		ImagePlus imp = WindowManager.getCurrentImage();
		if (imp==null) {
			error(msg_err_no_image);
			return null;
		} else
			return imp;
	}

	boolean runUroIrm(){
		int[] index = list.getSelectedIndexes();
		if ( index.length == 0)
			return error(msg_err_no_roi_selected);
		String s = "TITLE : ";
		s += list.getItem(index[0]) ;
		IJ.runPlugIn("UroIrmGraph_",s);
		return true;
	}

	boolean reName(){
		int[] index = list.getSelectedIndexes();
		if ( index.length == 0)
			return error(msg_err_no_roi_selected);
		if ( index.length > 1 ) 
			return error(msg_err_too_much_roi_selected);
		String s = list.getItem(index[0]) ;
		Roi roi = (Roi)rois.get(s);	
		rois.remove(s);
		String res = newNameDialog(s) ;
		res =(res == null)?s:res;
		list.replaceItem(res,index[0]);
		rois.put(res, roi.clone());
		return true;
	}

	String newNameDialog(String givenText){
		String res = showNewNameDialog(givenText) ;
		while (  (res !=null && (res.equals("") || rois.containsKey(res) )) ) {
			error(msg_err_name_exists);
			res = newNameDialog(givenText);
		}
		return res;
	}

	String showNewNameDialog(String givenText ){
		GenericDialog gd = new GenericDialog("Roi name");
		String label = givenText;
		gd.addStringField(msg_question_give_name, label ,label.length());
		gd.showDialog();
		if ( gd.wasCanceled())
			return null;
		return gd.getNextString();
	}

	boolean error(String msg) {
		new MessageDialog(this, "ROI Manager", msg);
		return false;
	}
	
	public void processWindowEvent(WindowEvent e) {
		super.processWindowEvent(e);
		if (e.getID()==WindowEvent.WINDOW_CLOSING) {
			instance = null;	
		}
	}

}

