package UroIrmPackage;

import ij.*;
import java.util.*;

/** create the different string for the title dicom stack and for the image 
	of this stack 
	This class can handle ImagePlus as image or as sack, but some fonction are
	specific for the stack and others for the images */
public class HeaderDicom implements Header,UroIrmMessage{
	ImageStack stack;
	ImagePlus imp;
	String tags ;
	String headerSlice;
	String headerStack;
	int currentSlice;
	boolean isStack;

	private String namePlugIn = "HeaderDicom";
	
	public HeaderDicom( ImagePlus impSource){
		this.imp=impSource;
		if (this.imp != null){
			if (imp.getStackSize()>1) {
				stack = imp.getStack();
				headerStack = imp.getTitle();
				currentSlice = imp.getCurrentSlice();
				headerSlice = stack.getSliceLabel(currentSlice); 
				tags = null;
				isStack=true;
			} else {
				stack = null;
				isStack=false;
				currentSlice = -1;
				headerStack = null;
				tags = (String)imp.getProperty("Info");
      			headerSlice = "";
				if ( tags == null)
					error(msg_err_format);
			}
		}	
	}


// ===================== NO STACK ==================================
/** add a tag form to the header of the current image*/
	public boolean addTag(String currentTag){
		if ( headerSlice == null)
			headerSlice="";
		if ( currentTag.equals(""))
			return true;
		for (int i =0 ; i< SLICE_TAG.length ; i++ ){
			if ( currentTag.equals(SLICE_TAG[i]) ) {
				headerSlice += SLICE_SEPARATOR +  SLICE_TAG[i] ;
				setTag(SLICE_TAG[i]);
				return true;
			}
		}
		return false;
	}

/** add a tag form to the header of the current image*/
	public boolean addTagStack(String currentTag){
		if ( headerStack == null ) 
			headerStack = STACK;
		if ( currentTag.equals(""))
			return true;
		for (int i =0 ; i< STACK_TAG.length ; i++ ){
			if ( currentTag.equals(STACK_TAG[i]) ) {
				headerStack+= STACK_SEPARATOR + STACK_TAG[i] ;
				headerStack+= getTagImage(STACK_TAG[i]);	
				return true;
			}
		}
		return false;
	}

	private void setTag( String tag){
		if ( tag.equals(ETAT))
			setEtat(ENABLE);
		else if ( tag.equals(TITLE))
			headerSlice +=imp.getTitle();
		else
			headerSlice +=getTagImage(tag);
	}

	private String[] getValueTag(String tag){
		String[] result = new String[2] ;
		result[0] = TYPE_GENERAL_TAG ;
		for (int i=0;i< GENERAL_TAG.length;i++){
			if ( tag.equals(GENERAL_TAG[i][0])) {
				result[1] =GENERAL_TAG[i][1];
				return result;		
			}
		}
		result[0]= TYPE_DATE_TAG;
		for (int i=0;i< DATE_TAG.length;i++){
			if ( tag.equals(DATE_TAG[i][0])){
				result[1] = DATE_TAG[i][1];
				return result;		
			}
		}
		result[0] = TYPE_TIME_TAG ;
		for (int i=0;i< TIME_TAG.length;i++){
			if ( tag.equals(TIME_TAG[i][0])){
				result[1] = TIME_TAG[i][1];
				return result;		
			}
		}
		return null;
	}

/** gets the information in the header of a DICOM image 
	if there is no information found, returns ""; 
	if it is a stack, returns null  */
	public String getTagImage ( String name ) {
		String S_ligne_nom = "" ;
		String S_nom = "" ;
		String S_diff = null;

		if ( isStack) 
			return null;
		try {	
			String[] result = getValueTag(name);
			if (result == null)
				return null;
			if ( result[0].equals(TYPE_TIME_TAG) ) {
				UroIrmTime time ;
				if ( result[1].equals("diff")) {
					time = getAcqTime();
					if ( time == null )
						time = new UroIrmTime(-1);
				} else{ 
					S_ligne_nom += selection(result[1]);
					S_nom += getChamp(S_ligne_nom);
					int seconde = tpsString2time(S_nom);
					time = new UroIrmTime(seconde);
				}
				return(time.toString());
			}
			
			S_ligne_nom += selection(result[1]);
			S_nom += getChamp(S_ligne_nom);
			S_nom = (S_nom.equals("null")?"":S_nom);
			if ( result[0].equals(TYPE_GENERAL_TAG) ) return (S_nom) ;
			if ( result[0].equals(TYPE_DATE_TAG) ) return (convertDate(S_nom)) ;
		} catch ( NullPointerException NPE  ) {
			error(msg_err_prgm + "\n" + msg_err_null_pointer + "\n" + msg_err_champ);	
		}
		return null;
	}

/** gets the information in the header of a DICOM image of a specific key 
		type = 1 <=> date , type = 2 <=> time, default  <=> others 
*/
	public String key2TagImage ( String key, int type) {
		String s_ligne  ;
		String s  ;
		String res;

		if ( isStack) 
			return null;
		s_ligne = selection(key);
		s = getChamp(s_ligne);
		switch (type){
			case 1 :
				res = convertDate(s) ;
		 		break;
			case 2 :
				int seconde = tpsString2time(s);
				if ( seconde != -1 )
					res = ((new UroIrmTime(seconde)).toString());
				else 
					res ="";
				break;
			default : 
				res =  s;
				break;
		} 	
		if (res == null || res.equals("") )
			return("Not found");
		return res;
	}

/** put the date in a better presentation form : 
	returns the date as : day / month / year
*/ 
	public String convertDate(String date){
		if ( date == null ) 
			return "";
		if ( date.length() != 8 )
			return "";
		String year = (date.substring(0,4));
		String month = (date.substring(4,6));
		String day = (date.substring(6,8));
		return ( day + "/" + month + "/" + year);
	}

/** get the acquisition time ( which is the difference between the time of the examen
	and the time of the image ). 
*/
	public UroIrmTime getAcqTime() {
		if ( isStack ) 
			return null;
		try {
			String S_ligne_imageTps = selection("0008,0033");
			String S_ligne_acqTps = selection("0008,0032");
			String S_imageTps = getChamp(S_ligne_imageTps);
			String S_acqTps = getChamp(S_ligne_acqTps);
			int imageTps =  tpsString2time(S_imageTps);
			int acqTps = tpsString2time(S_acqTps);
			if ( acqTps <0 ||imageTps <0 )
				return null;
			UroIrmTime resultat = new UroIrmTime (imageTps - acqTps);
			
			return(resultat);

		} catch ( NullPointerException NPE  ) {
			error(msg_err_prgm + "\n" + msg_err_null_pointer + "\n" + msg_err_champ);	
		}
		return null;
	} 


	private String selection( String key){
		String source = this.tags;
		int indice_key = -1 ;
		int indice_fin = -1;
		
		if ( isStack)
			return null;	
		indice_key = source.indexOf(key);
		if (indice_key < 0 ){
			return "";
		}
		indice_fin = source.indexOf("\n",indice_key);
		if (indice_fin < indice_key) {
			error(msg_err_prgm);
			return null;
		}	
		return source.substring(indice_key,indice_fin);
	}


	private String getChamp (String source) {
		int indice_debut =0;
		int res =-1;		
		String entier;		

		if ( source ==null)
			return null;
		indice_debut  = source.indexOf(":")  ; 	
		if ( indice_debut < 0 ) { return null;}
		indice_debut +=2;		
		return( source.substring(indice_debut) ) ;
	}

/** put the time in a better presentation :it converts HHMMSS  in seconde (int) */
	public int tpsString2time(String tpsString){
		int indice_point =-1;	
		int intermediaire = 0;
		int heure,minute,seconde,resultat ;
		
		heure =0; minute =0 ; seconde =0 ; resultat=0;
		try {
			if (tpsString ==null)
				return -1;
			indice_point = tpsString.indexOf(".");
			if ( indice_point < 0 ) { 
				intermediaire = Integer.parseInt(tpsString);
			} else {
				intermediaire = Integer.parseInt(tpsString.substring(0,indice_point));
			}
			heure = intermediaire / 10000; // pour les heures
			minute = (intermediaire - heure * 10000 ) / 100 ;
			seconde = (intermediaire - heure * 10000 - minute *100 );
			resultat = heure * 3600 + minute *60 + seconde ;		
			return resultat;
		}catch (NumberFormatException e ) {
			//error(" tpsString2time " + msg_err_NFE);
			return -1;
		}
	}

// ==============================  STACK REQUIRED ==============================
/** set the image of the specific index "n" of the stack */
	public boolean setSlice(int n){
		if ( !isStack)
			return false;
		if ( n > imp.getStackSize() || n < 0 )
			return false;
		headerSlice = stack.getSliceLabel(n);
		currentSlice = n;
		return true;
	}

/** put the header on the Nth images of the stack or on one image*/
	public void DisplaySlice(){
		if ( !isStack )
			return;
		stack.setSliceLabel(headerSlice,currentSlice);
		imp.repaintWindow();
	}

/** set the title of the stack */
	public void DisplayStack(){
		if ( !isStack) 
			return;
		imp.setTitle(headerStack);
		imp.repaintWindow();
	}
	
		
/** search a specific tag of the header, true if it exists otherwise false*/
	public boolean isTag(String tag ){
		String label="";
		label += headerSlice;
		int index = label.indexOf(tag);
		if (index > 0 )
			return true ;
		return false;
	}

/** add a tag form to the header of a stack*/
	public String addTagSlice2Stack(String tag){
		if (!isStack)
			return null;
		String label = "";
		label +=headerStack;
		for (int i =0 ; i< STACK_TAG.length ; i++ ){
			if ( tag.equals(STACK_TAG[i]) ) {
				String currentTag = "";
				currentTag += getTagValue(tag);
				if (currentTag == "")
					break;
				label+= STACK_SEPARATOR + tag + currentTag;
				break;
			}
		}
		return label;
	}

/** find the value of the tag in the header of the n image in a stack*/
	public String getTagValue(String tag) {
		String label_title = "";
 		
		if ( !isStack )
			return null;
		label_title +=  headerStack + STACK_SEPARATOR +  headerSlice;
		int index = label_title.indexOf(tag);
		String label_unfinish  = "";				
		String label ="";
		if (index > 0 ){
			index+=tag.length();
			label_unfinish+=label_title.substring(index);				
		}
		index = label_unfinish.indexOf(SLICE_SEPARATOR);
		if (index > 0 ) 
			label += label_unfinish.substring(0,index);
		 else 
			label += label_unfinish;
		if ( tag == ACQTIME ) {
			if (label==null || label.equals(""))
				label = UroIrmTime.ANORMAL;
		}		
		return label;
	}

/** fonction that switch the label ETAT to ENABLE/DISABLE .
	this label ETAT must exist*/
	public boolean switchEtat(){
		return(stateEtat("",true) );
	}

/** fonction that set the label ETAT to "status" 
	if the label ETAT doesn't exist, this fonction add it with the "status" */
	public boolean setEtat(String status){
		return (stateEtat(status,false));
	}

	private boolean stateEtat(String status, boolean change) {
		if ( !isStack )
			return false;
		String etat = getTagValue(ETAT) ;  
		String title = headerSlice;
		if ( change && (etat == null || etat.equals("")) ){
			return false;
		}
		if ( !change && (etat == null || etat.equals("")) ){
			headerSlice += SLICE_SEPARATOR + ETAT + status;
			DisplaySlice();
			return true;
		}	
		String before_Etat = title.substring(0,title.indexOf(ETAT));
		String after_Etat = "";
		int index = (title.substring(title.indexOf(ETAT))).indexOf(SLICE_SEPARATOR) ;
		if ( index > 0)
				after_Etat = title.substring(index);
		if ( change ) {
			if ( etat.equals(ENABLE) )
				title = before_Etat + ETAT + DISABLE + after_Etat;
			else	if ( etat.equals(DISABLE) ) 
				title = before_Etat + ETAT + ENABLE + after_Etat;
		} else {
			if ( status.equals(ENABLE) )
				title = before_Etat + ETAT + ENABLE + after_Etat;
			else	if ( status.equals(DISABLE) ) 
				title = before_Etat + ETAT + DISABLE + after_Etat;
		}	
		headerSlice = title;		
		DisplaySlice();
		return true;
	}




// ============================== BOTH OF THEM =================================

/** get the header of the Nth images of the stack or on one image	*/
	public String getSlice() {
		return(headerSlice);
	}

	public String getStack(){
		return headerStack;
	}

	private String[] convertTokenizer2Array(StringTokenizer chaine){
		if ( chaine.countTokens() <= 0 )
			return null;
		String[] result = new String[chaine.countTokens()];
		for (int i =0; i< result.length;i++){
			result[i] = chaine.nextToken();
		}
		return result;
	}

	public String[] getArraySlice(){
		StringTokenizer chaine = new StringTokenizer(headerSlice,SLICE_SEPARATOR);
		return convertTokenizer2Array(chaine);		
	}

	public String[] getArrayStack(){
		StringTokenizer chaine = new StringTokenizer(headerStack,STACK_SEPARATOR);
		return convertTokenizer2Array(chaine);	
	}

	private boolean error(String txt){
		IJ.error(namePlugIn + "\n" + txt);
		return false;
	}

// for debug
	private void showInfo( String comm , int n){
		String text ="";
		text += " stack : ";
		if ( stack !=null)
			text += "YES " ;
		else 
			text+="NO";
		text += " ; headerStack =  " + headerStack ;
		text +=" ; isStack = ";
		if ( isStack )
			text +=" true";
		else
			text+=" false";
		text += " ; headerSlice = " + headerSlice;
		text += " ; tags : ";
		if ( tags == null )
			text += " nul ";
		else if ( tags.equals(""))
			text +=" none";
		else 
			text += "YES";
		text += "; " +  comm;
		
		IJ.write(text);
	}



}
